local socket = require "socket"

server = {}

local data, ip, port
local cmd
local timer = 0
netplay = false
local resynctimer = -5
local resyncconfig = false 

common_chars = {}
clients = {}

local syncnow = true
local synctimer = 0
local synctimemax = 0.03
local shake = "Turtle-Invaders"

function server:load(port)
	netplay = true

	self.udp = socket.udp()

	self.udp:settimeout(0)
	self.udp:setsockname('*', port)
end

function server:update(dt)
	data, ip, port = self.udp:receivefrom()
	if data then
	    timer = 0

	    cmd = data:split(";")

	    if cmd[1] == "connect" then
	    	if #clients < 4 then
		        server:addClient(cmd[2], cmd[3], cmd[4], ip, port)
		        self.udp:sendto("connected", ip, port)
				self.udp:sendto("clientnumber;" .. #clients, ip, port)
				self.udp:sendto("playsound;coopconnect;", ip, port)

		        resynctimer = 0
		    else
	        	self.udp:sendto("lobbyfull;", ip, port)
	        	return
	        end
	    elseif cmd[1] == "chars" then
	    	server:compareChars(cmd)
	    elseif cmd[1] == "chat" then
	    	server:parseChat(cmd, ip)
	    elseif cmd[1] == "closing" then
			if cmd[4] == "true" then
				server:sendDataToClients("disconnect;Server shutting down!;true", ip)

				--remove the host, make sure they disconnect you dolt!
				self.udp:sendto("disconnect;", ip, port)

				for k = 1, #clients do
					table.remove(clients, k)
				end

				self.udp:close()

				netplay = false
			else
				server:sendDataToClients("playerleft;" .. cmd[2] .. ";" .. cmd[3] .. ";")
				
				server:removeClient(cmd)

				resyncconfig = true
			end
		elseif cmd[1] == "begingame" then
			
			local t = ""
			for k = 1, #clients do
				t = t .. clients[k].character .. ";"
			end

			server:sendNotify(cmd[1] .. ";" .. #clients .. ";" .. t, ip)
		elseif cmd[1] == "batboss" then
			server:sendDataToClients(data, ip)
		elseif cmd[1] == "coonboss" then
			server:sendDataToClients(data, ip)
		elseif cmd[1] == "bossshot" then
			server:sendDataToClients(data, ip)
		elseif cmd[1] == "batshot" then
			server:sendDataToClients(data, ip)
		elseif cmd[1] == "bosshit" then
			server:sendDataToClients(data, ip)
		elseif cmd[1] == "phoenixboss" then
			server:sendDataToClients(data, ip)
		elseif cmd[1] == "phoenixshot" then
			server:sendDataToClients(data, ip)
		elseif cmd[1] == "phoenixply" then
			server:sendDataToClients(data, ip)
		elseif cmd[1] == "phoenixshieldshot" then
			server:sendDataToClients(data, ip)
		end

		server:syncPlayers(cmd, ip)

		server:syncBats(data, ip)

		if cmd[1] == "spawnenemy" then
			server:sendDataToClients(data, ip)

			return
		end	

		timer = 0
	elseif ip ~= 'timeout' then
    	newNotice("Unknown network error: " .. tostring(ip))

    	self.udp:close()
    	netplay = false
    elseif ip == "timeout" then
	   
	elseif ip == "closing" then
		server:sendDataToClients("disconnect;Server shutting down!;true", ip)

		--remove the host, make sure they disconnect you dolt!
		self.udp:sendto("disconnect;", ip, port)

		for k = 1, #clients do
			table.remove(clients, k)
		end
		
		self.udp:close()

		netplay = false
	end

	if resynctimer >= 0 then
		resynctimer = resynctimer + dt
		if resynctimer > .4 then
			resynctimer = -5
			resyncconfig = true
		end
	end

	if resyncconfig then
		server:syncData()
		resyncconfig = false
	end
end

function server:removeClient(cmd, ip)
	if lobby_playerlist[tonumber(cmd[2])] then
		table.remove(lobby_playerlist, tonumber(cmd[2]))
	end

	table.remove(clients, tonumber(cmd[2]))
				
	playsound("coopdisconnect")

	newNotice("Client " .. cmd[3] .. " has disconnected.")
end

function server:syncPlayers(cmd, ip)
	if cmd[1] == "playerinfo" then
		server:sendDataToClients("playerinfo;" .. cmd[2] .. ";" .. cmd[3] .. ";" .. cmd[4] .. ";", ip)
	elseif cmd[1] == "shoot" then
		server:sendDataToClients("shoot;" .. cmd[2] .. ";" .. cmd[3], ip)
	elseif cmd[1] == "powerup" then
		server:sendDataToClients("powerup;" .. cmd[2] .. ";" .. cmd[3], ip)
	elseif cmd[1] == "bullettype" then
		server:sendDataToClients("bullettype;" .. cmd[2] .. ";" .. cmd[3], ip)
	elseif cmd[1] == "health" then
		server:sendDataToClients("health;" .. cmd[2] .. ";" .. cmd[3], ip)
	elseif cmd[1] == "specialability" then
		server:sendDataToClients("specialability;" .. cmd[2] .. ";")
	elseif cmd[1] == "specialabilitykeypress" then
		server:sendDataToClients("specialabilitykeypress;" .. cmd[2] .. ";")
	end
end

function server:syncBats(data, ip)
	local cmd = data:split(";")

	if cmd[1] == "batdata" then
		server:sendDataToClients(data, ip)
	elseif cmd[1] == "batdied" then
		server:sendDataToClients(data, ip)
	elseif cmd[1] == "powerspawn" then
		server:sendDataToClients(data, ip)
	end
end

function server:syncData()
	for k, s in ipairs(clients) do
		for j, v in pairs(clients) do
			if j ~= k then --self.udp:sendto("playerdata;" .. k .. ";" .. v.nickname .. ";" .. v.character .. ";" .. v.hosting .. ";", v.ip, v.port)
				self.udp:sendto("playerdata;" .. k .. ";" .. s.nickname .. ";" .. s.character .. ";" .. s.hosting .. ";", v.ip, v.port)
			end
		end
	end
end

function server:addClient(nick, char, host, ip, port)
	table.insert(clients, {nickname = nick, character = char, hosting = host, ip = ip, port = port})

	newNotice("Client " .. nick .. " has joined.")

	if not host then
		playsound("coopconnect")
	end

	--server:sendDataToClients("connectionsuccessful;", ip)
end

function server:parseChat(cmd, ip)
	local chattext = cmd[2]

	server:sendDataToClients("chat;" .. cmd[2], ip)
end

function server:getHostName()
	return nickstr
end

function server:getCharName()
	return gamechars[chari]
end

function server:parseBatAbilities(cmd, ip)
	local a = cmd[2]
	local b = cmd[3]

	server:sendDataToClients("batabilities;" .. cmd[1] .. ";" .. a .. ";" .. b, ip)
end

function server:sendNotify(cmd, ip)
	players = #clients
	difficulty = difficultytypes[gameDifficulty]
	gamemode = gamemodes[gameModei]

	nameData = {}
	local addon = ""
	for k = 1, #clients do
		addon = addon .. clients[k].nickname .. ";"
	end

	server:sendDataToClients(cmd, ip)
	server:sendDataToClients("names;" .. addon .. gameDifficulty, ip)
	server:sendDataToClients("gameSetup;" .. gameDifficulty .. ";" .. gameModei .. ";", ip)

	for k = 1, #clients do
		table.insert(gameData, clients[k].character)
		table.insert(nameData, clients[k].nickname)
	end

	shared:timeEvent(1, game_load)
end

function server:sendDataToClients(string, ip)
	for k, v in ipairs(clients) do
		if ip ~= v.ip then
			self.udp:sendto(string, v.ip, v.port)
		end
	end
end